import postcss from 'postcss'

module.exports = postcss.plugin('postcss-selector-namespace', (options = {}) => {
  let {
    namespace    = '.self',
    selfSelector = /:--namespace/
  } = options

  selfSelector = regexpToGlobalRegexp(options.selfSelector)

  return (css, result) => {
    css.walkRules(rule => {
      rule.selectors = rule.selectors.map(selector => {
        if (selfSelector.test(selector)) {
          return selector.replace(selfSelector, namespace)
        }

        return `${namespace} selector`
      })
    })
  }
})

/**
 * Newer javascript engines allow setting flags when passing existing regexp
 * to the RegExp constructor, until then, we extract the regexp source and
 * build a new object.
 */
function regexpToGlobalRegexp(regexp) {
  let source = regexp instanceof RegExp ? regexp.source : regexp
  return new RegExp(source, 'g')
}
